/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package studentcollection;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * Seznam studentů s procházením řešeným pomocí ListIterator
 * @author balikm1
 */
public class SeznamStudentu {
    // kolekce List
    private List<Student> studentList;
    // Iterátor
    ListIterator<Student> i;
    // aktuálně vybraný student
    private Student current;
    
    public SeznamStudentu() {
        // vytvoříme nový List
        studentList = new ArrayList<Student>();
        // získáme iterátor
        i = studentList.listIterator();
    }
    
    /**
     * Přidá studenta na aktuální pozici iterátoru
     * @param s - student
     */
    public void pridej(Student s) {
        // nutné přidávat metodou iterátoru, jinak kdybychom změnili list pomocí list.add, iterátor by přestal fungovat
        // přidává záznam na aktuální pozici kurzoru a kurzor se posune za tento záznam
        i.add(s);
        // záznam je před kurzorem, zavolame tedy predchozi (napr. remove je zavisla na zaznamu naposledy vracenym getNext/getPrevious
        predchozi();
        // abychom meli kurzor za zaznamem, zavolame predchozi,dalsi aby posledni operace nad iteratorem byla getNext
        predchozi();dalsi();
    }
    
    /**
     * Smaže studenta na aktuální pozici iterátoru
     */
    public void smaz() {
        if (current != null) {
            // pokud current neni null, muzeme zaznam mazat, do current se nam objekt dostava pouze volanim getNext/getPrevious, tedy metoda muze byt pouzita 
            i.remove();
            // po smazani se pokusime prejit na dalsi zaznam
            if (i.hasNext())
                dalsi();
            // pokud neexistuje, prejdeme na predchozi
            else if (i.hasPrevious())
                predchozi();
            // pokud take neexistuje jiz zadny zaznam nemame
            else
                current = null;
        }
    }
    
    /**
     * Zjistí, zda seznam obsahuje studenta s
     * @param s - student
     */
    public void obsahuje(Student s) {
        studentList.contains(s);
    }
    
    /**
     * Vrátí iterátor seznam
     * @return iterátor
     */
    public Iterator getIterator() {
        return studentList.iterator();
    }
    
    private void dalsi() {
        Student s = null;
        // při změně směru iterátoru se stane, že dostaneme stejný záznam
        //   S1 S2
        // ^     
        // poslední getPrevious vrátilo S1, kurzor se nachází před S1. Následné getNext vrátí S1, ale my chceme raději S2
        while (i.hasNext() && (s = i.next()).equals(current)) ;
        if (s != null)
            current = s;
    }
    
    private void predchozi() {
        Student s = null;
        // při změně směru iterátoru se stane, že dostaneme stejný záznam
        // S1 S2
        //       ^
        // poslední getNext vrátilo S2, kurzor se nachází za S2. Následné getPrevious vrátí S2, ale my chceme raději S1
        while (i.hasPrevious() && (s = i.previous()).equals(current)) ;
        if (s != null)
            current = s;
    }
    
    /**
     * Vrátí aktuálně vybraného studenta
     * @return student
     */
    public Student getCurrent() {
        return current;
    }
    
    /**
     * Vrátí dalšího studenta dle pozice iterátoru
     * @return student
     */
    public Student getDalsi() {
        dalsi();
        return getCurrent();
    }
    
    /**
     * Vrátí předchozího studenta dle pozice iterátoru
     * @return student
     */
    public Student getPredchozi() {
        predchozi();
        return getCurrent();
    }
    
    /**
     * Vrátí zda existuje další student dle pozice iterátoru
     * @return true pokud existuje další
     */
    public boolean hasDalsi() {
        return i.hasNext();
    }
    
    /**
     * Vrátí zda existuje předchozí student dle pozice iterátoru
     * @return true pokud existuje předchozí
     */
    public boolean hasPredchozi() {
        return i.hasPrevious();
    }
}
